package net.wanji.datacenter.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.kafka.ConsumerHandler;
import net.wanji.datacenter.mapper.*;
import net.wanji.datacenter.pojo.convert.LanePeriodicDataEventListConvert;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.pojo.po.*;
import net.wanji.datacenter.service.DataProcessService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/10 10:30
 * @description: 车道周期实时数据加工处理服务，优化模块实时监控操作数据
 */
@Slf4j
@Service(value = "JN040001LanePeriodicData")
@RequiredArgsConstructor
@SuppressWarnings("all")
public class LanePeriodicDataProcessServiceImpl implements DataProcessService {

    private final ConsumerHandler consumerHandler;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossDirDataRealTimeMapper crossDirDataRealTimeMapper;
    private final CrossTurnDataRealTimeMapper crossTurnDataRealTimeMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossDataRealTimeMapper crossDataRealTimeMapper;
    private final LanePeriodicDataEventListConvert lanePeriodicDataEventListConvert;

    @Override
    public String receive(String topic) {
        return consumerHandler.getTopicMessage(topic);
    }

    @Override
    public void send(String topic, Object convertData) {
        consumerHandler.removeTopicMessage(topic);
    }

    @Override
    public Object convert(String originalData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataPO = null;
        try {
            lanePeriodicDataPO = JSONObject.parseObject(originalData, LanePeriodicDataDTO.class);
        } catch (Exception e) {
            log.error("车道周期实时数据转换异常", e);
            throw new Exception();
        }
        return lanePeriodicDataPO;
    }

    @Override
    @Transactional
    public void save(Object convertData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataDTO = (LanePeriodicDataDTO) convertData;

        int laneNum = lanePeriodicDataDTO.getLaneNum();
        List<CrossDataRealTimePO> crossDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossDirDataRealTimePO> crossDirDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossTurnDataRealTimePO> crossTurnDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = new ArrayList<>(laneNum);

        List<LanePeriodicDataDTO.EventList> eventLists = lanePeriodicDataDTO.getEventList();
        if (CollectionUtils.isEmpty(eventLists)) {
            log.error("当前车道周期实时数据车道列表为空,参数信息：", lanePeriodicDataDTO);
            throw new Exception("当前车道周期实时数据车道列表为空");
        }
        // 通过车道编号获取车道信息
//        List<String> LaneIdList = eventLists.stream().map(LanePeriodicDataDTO.EventList::getLaneId).collect(Collectors.toList());
        List<String> LaneIdList = Arrays.asList("14PU609IQA014PUG09IRA0010011", "14PU609IQA014PUG09IRA0010012");
        List<CrossBaseLaneInfoPO> crossBaseLaneInfoPOS = crossBaseLaneInfoMapper.selectBatchIds(LaneIdList);

        // 将kafka车道数据与路口车道数据对应，获取路口编号，方向，转向，分组计算
        for (LanePeriodicDataDTO.EventList laneRealTimeItem : eventLists) {
            String laneId = laneRealTimeItem.getLaneId();
            for (CrossBaseLaneInfoPO laneInfoPO : crossBaseLaneInfoPOS) {
                String id = laneInfoPO.getId();
                if (StringUtils.endsWithIgnoreCase(laneId, id)) {

                    crossDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossDirDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossDirDataRealTimePO(laneRealTimeItem, laneInfoPO));

                    crossTurnDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossTurnDataRealtimePO(laneRealTimeItem, laneInfoPO));

                    crossLaneDataRealTimePOS.add(lanePeriodicDataEventListConvert.convert2CrossLaneDataRealTimePO(laneRealTimeItem, laneInfoPO));
                }
            }
        }
        // 删除操作 实时数据表只能有一条实时数据
        Set<String> crossDataRealTimeSet = crossDataRealTimePOS.stream().map(CrossDataRealTimePO::getCrossId).collect(Collectors.toSet());
        crossDataRealTimeMapper.deleteBatch(crossDataRealTimeSet);

        Set<String> crossDirDataRealTimeSet = crossDirDataRealTimePOS.stream().map(CrossDirDataRealTimePO::getCrossId).collect(Collectors.toSet());
        crossDirDataRealTimeMapper.deleteBatch(crossDirDataRealTimeSet);

        Set<String> crossTurnDataRealTimeSet = crossTurnDataRealTimePOS.stream().map(CrossTurnDataRealTimePO::getCrossId).collect(Collectors.toSet());
        crossTurnDataRealTimeMapper.deleteBatch(crossTurnDataRealTimeSet);

        Set<String> crossLaneDataRealTimeSet = crossLaneDataRealTimePOS.stream().map(CrossLaneDataRealTimePO::getCrossId).collect(Collectors.toSet());
        crossLaneDataRealTimeMapper.deleteBatch(crossLaneDataRealTimeSet);

        // 将车道转换的数据通过路口，方向，转向维度计算，存储
        List<CrossDataRealTimePO> insertCrossDataRealTimePOS = new ArrayList<>();
        Map<String, List<CrossDataRealTimePO>> crossDataRealTimePOSMap = crossDataRealTimePOS.stream().collect(Collectors.groupingBy(CrossDataRealTimePO::getCrossId));
        for (Map.Entry<String, List<CrossDataRealTimePO>> entry : crossDataRealTimePOSMap.entrySet()) {
            String crossId = entry.getKey();
            List<CrossDataRealTimePO> value = entry.getValue();
            CrossDataRealTimePO insertCrossDataRealTimePO = new CrossDataRealTimePO();
            Integer status = 0;
            Double trafficIndex = 0.0;
            Integer flow = 0;
            Double flowRate = 0.0;
            Double speed = 0.0;
            Double queueLength = 0.0;
            Integer stopTimes = 0;
            Integer delayTime = 0;
            for (CrossDataRealTimePO crossDataRealTimePO : value) {
                // 每个路口计算交通流量累加，筛选最大停车长度
                flow += crossDataRealTimePO.getFlow();
                flowRate += crossDataRealTimePO.getFlowRate();
                speed += crossDataRealTimePO.getSpeed();
                queueLength = queueLength >= crossDataRealTimePO.getQueueLength() ? queueLength : crossDataRealTimePO.getQueueLength();
                stopTimes += crossDataRealTimePO.getStopTimes();
                delayTime += crossDataRealTimePO.getDelayTime();
                BeanUtils.copyProperties(crossDataRealTimePO, insertCrossDataRealTimePO);
            }
            insertCrossDataRealTimePO.setStatus(status);
            insertCrossDataRealTimePO.setTrafficIndex(trafficIndex);
            insertCrossDataRealTimePO.setFlow(flow);
            insertCrossDataRealTimePO.setFlowRate(flowRate / value.size());
            insertCrossDataRealTimePO.setSpeed(speed / value.size());
            insertCrossDataRealTimePO.setQueueLength(queueLength);
            insertCrossDataRealTimePO.setStopTimes(stopTimes / value.size());
            insertCrossDataRealTimePO.setDelayTime(delayTime / value.size());
            insertCrossDataRealTimePOS.add(insertCrossDataRealTimePO);
        }
//        crossDataRealTimeMapper.insertBatch(insertCrossDataRealTimePOS);

        // 方向数据
        List<CrossDirDataRealTimePO> insertCrossDirDataRealTimePOS = new ArrayList<>();
        Map<String, List<CrossDirDataRealTimePO>> crossDirDataRealTimePOSMap = crossDirDataRealTimePOS.stream().collect(Collectors.groupingBy(CrossDirDataRealTimePO::getCrossId));
        for (Map.Entry<String, List<CrossDirDataRealTimePO>> entry : crossDirDataRealTimePOSMap.entrySet()) {
            String key = entry.getKey();
            List<CrossDirDataRealTimePO> value = entry.getValue();
            Map<Integer, List<CrossDirDataRealTimePO>> crossIdMap = value.stream().collect(Collectors.groupingBy(CrossDirDataRealTimePO::getDirType));

            CrossDirDataRealTimePO crossDirDataRealTimePO = new CrossDirDataRealTimePO();
            Integer status = 0;
            Double trafficIndex = 0.0;
            Integer flow = 0;
            Double flowRate = 0.0;
            Double speed = 0.0;
            Double queueLength = 0.0;
            Integer stopTimes = 0;
            Integer delayTime = 0;
            for (Map.Entry<Integer, List<CrossDirDataRealTimePO>> dirEntry : crossIdMap.entrySet()) {
                Integer dir = dirEntry.getKey();
                List<CrossDirDataRealTimePO> dirMap = dirEntry.getValue();
                // 每个方向计算交通流量累加，筛选最大停车长度
            }
            insertCrossDirDataRealTimePOS.add(crossDirDataRealTimePO);
        }
//        crossDirDataRealTimeMapper.insertBatch(insertCrossDirDataRealTimePOS);

        // 转向数据
        ArrayList<CrossTurnDataRealTimePO> insertCrossTurnDataRealTimePOS = new ArrayList<>();
        Map<String, List<CrossTurnDataRealTimePO>> crossTurnDataRealTimePOSMap = crossTurnDataRealTimePOS.stream().collect(Collectors.groupingBy(CrossTurnDataRealTimePO::getCrossId));
        for (Map.Entry<String, List<CrossTurnDataRealTimePO>> entry : crossTurnDataRealTimePOSMap.entrySet()) {
            String key = entry.getKey();
            List<CrossTurnDataRealTimePO> value = entry.getValue();

            CrossTurnDataRealTimePO crossTurnDataRealTimePO = new CrossTurnDataRealTimePO();
            Map<String, List<CrossTurnDataRealTimePO>> crossIdMap = value.stream().collect(Collectors.groupingBy(CrossTurnDataRealTimePO::getTurnType));
            for (Map.Entry<String, List<CrossTurnDataRealTimePO>> item : crossIdMap.entrySet()) {
                String turn = item.getKey();
                List<CrossTurnDataRealTimePO> turnMap = item.getValue();
            }
            insertCrossTurnDataRealTimePOS.add(crossTurnDataRealTimePO);
        }
//        crossTurnDataRealTimeMapper.insertBatch(insertCrossTurnDataRealTimePOS);

        // 车道数据
//        crossLaneDataRealTimeMapper.insertBatch(crossLaneDataRealTimePOS);

    }
}
